"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withMutation = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _hoc = require("@apollo/client/react/hoc");

var _lodash = _interopRequireDefault(require("lodash"));

var _immutabilityHelper = _interopRequireDefault(require("immutability-helper"));

var _pcmliUmbrella = require("pcmli.umbrella.uni-core");

var getMethodName = function getMethodName(_ref) {
  var _type = _ref._type,
      methodName = _ref.methodName,
      actionName = _ref.actionName;
  var localMethodName = 'onUpdate',
      localActionName = 'Update';

  if (_type === 'create' || _type === 'insert') {
    localMethodName = 'onCreate';
    localActionName = 'Create';
  } else if (_type === 'delete') {
    localMethodName = 'onDelete';
    localActionName = 'Delete';
  } //but if there is no value for methodName and actionName, then use the local


  methodName = methodName || localMethodName;
  actionName = actionName || localActionName;
  return {
    methodName: methodName,
    actionName: actionName
  };
};

var buildUpdateQueries = function buildUpdateQueries(_ref2) {
  var mutationParams = _ref2.mutationParams,
      entity = _ref2.entity,
      queryName = _ref2.queryName,
      mutationName = _ref2.mutationName;
  var _type = mutationParams._type; //if query name was not specified then user the model name

  queryName = queryName || entity; //if mutation name was not specified then user the model name

  mutationName = mutationName || entity;
  if (_type === 'update') return null;
  var updateQueries = (0, _defineProperty2["default"])({}, queryName, function (previousResult, _ref3) {
    var mutationResult = _ref3.mutationResult;
    var mutateResult = mutationResult.data[mutationName];

    if (_type === 'delete') {
      var index = _lodash["default"].findIndex(previousResult[entity], {
        _id: mutateResult._id
      });

      if (index === -1) throw new Error('Cannot find the id');
      return (0, _immutabilityHelper["default"])(previousResult, (0, _defineProperty2["default"])({}, entity, {
        $splice: [[index, 1]]
      }));
    } // else if (_type == 'insert') {
    //   return update(previousResult, {[entity]: {$unshift: [mutateResult]}})
    // }

  });
  return updateQueries;
};

var defaultParamsToDoc = function defaultParamsToDoc(_ref4) {
  var params = _ref4.params;
  return params;
};

var withMutation = function withMutation(_ref5) {
  var _type = _ref5._type,
      _ref5$paramsToDoc = _ref5.paramsToDoc,
      paramsToDoc = _ref5$paramsToDoc === void 0 ? defaultParamsToDoc : _ref5$paramsToDoc,
      query = _ref5.query,
      entity = _ref5.entity,
      refetchQueries = _ref5.refetchQueries,
      before = _ref5.before,
      after = _ref5.after,
      methodName = _ref5.methodName,
      actionName = _ref5.actionName,
      extraOptions = _ref5.extraOptions,
      extraProps = _ref5.extraProps,
      paramsToFilter = _ref5.paramsToFilter;
  var getMethodNameResult = getMethodName({
    _type: _type,
    methodName: methodName,
    actionName: actionName
  });
  var mutationName = "$mutate{entity}";
  refetchQueries = refetchQueries || ["".concat(entity)];
  var graphqlConfig = {
    name: mutationName,
    props: function props(args) {
      var _result;

      var ownProps = args.ownProps; //if extraProps  is function then execute it passing the ownProps, else this is an object so just use it

      var xProps = (_lodash["default"].isFunction(extraProps) ? extraProps({
        ownProps: ownProps
      }) : extraProps) || {};
      var afterName = "after".concat(getMethodNameResult.actionName);
      var beforeName = "before".concat(getMethodNameResult.actionName);
      var result = (_result = {}, (0, _defineProperty2["default"])(_result, getMethodNameResult.methodName, function () {
        var _ref6 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(params) {
          var doc, filter, _id, docString, filterString, variables, optimisticResponse, update, getOptimisticResponse;

          return _regenerator["default"].wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  doc = {};

                  if (!paramsToDoc) {
                    _context2.next = 5;
                    break;
                  }

                  _context2.next = 4;
                  return paramsToDoc({
                    params: params,
                    ownProps: ownProps
                  });

                case 4:
                  doc = _context2.sent;

                case 5:
                  filter = _lodash["default"].get(params, 'filter') || _lodash["default"].get(ownProps, 'filter');
                  _id = _lodash["default"].get(doc, '_id') || _lodash["default"].get(params, '_id');

                  if (paramsToFilter) {
                    filter = paramsToFilter({
                      params: params,
                      ownProps: ownProps
                    });
                  } //TODO: This should be tested


                  if (!filter) {
                    //if filter is not available then build it using the _id, this is required anyway for mutation
                    filter = {
                      _id: _id
                    };
                  } //TODO: Should be tested


                  doc = _lodash["default"].omit(doc, ['_id']); //should remove _id from doc object

                  docString = JSON.stringify(doc);
                  filterString = filter && JSON.stringify(filter); //get the id from doc, if it dies not exists there, then get the value from the params

                  variables = {
                    _id: _id,
                    _type: _type,
                    input: docString,
                    filter: filterString
                  };
                  optimisticResponse = xProps.optimisticResponse, update = xProps.update;

                  getOptimisticResponse = /*#__PURE__*/function () {
                    var _ref8 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(_ref7) {
                      var optimisticResponse;
                      return _regenerator["default"].wrap(function _callee$(_context) {
                        while (1) {
                          switch (_context.prev = _context.next) {
                            case 0:
                              optimisticResponse = _ref7.optimisticResponse;

                              if (!_lodash["default"].isFunction(optimisticResponse)) {
                                _context.next = 11;
                                break;
                              }

                              if (!_pcmliUmbrella.Utils.isPromise(optimisticResponse)) {
                                _context.next = 8;
                                break;
                              }

                              _context.next = 5;
                              return optimisticResponse({
                                ownProps: ownProps,
                                doc: doc
                              });

                            case 5:
                              return _context.abrupt("return", _context.sent);

                            case 8:
                              return _context.abrupt("return", optimisticResponse({
                                ownProps: ownProps,
                                doc: doc
                              }));

                            case 9:
                              _context.next = 12;
                              break;

                            case 11:
                              return _context.abrupt("return", optimisticResponse);

                            case 12:
                            case "end":
                              return _context.stop();
                          }
                        }
                      }, _callee);
                    }));

                    return function getOptimisticResponse(_x2) {
                      return _ref8.apply(this, arguments);
                    };
                  }();

                  _context2.next = 17;
                  return getOptimisticResponse({
                    optimisticResponse: optimisticResponse
                  });

                case 17:
                  _context2.t0 = _context2.sent;

                  if (_context2.t0) {
                    _context2.next = 20;
                    break;
                  }

                  _context2.t0 = undefined;

                case 20:
                  optimisticResponse = _context2.t0;
                  return _context2.abrupt("return", args[mutationName]({
                    variables: variables,
                    optimisticResponse: optimisticResponse,
                    update: update
                  }));

                case 22:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2);
        }));

        return function (_x) {
          return _ref6.apply(this, arguments);
        };
      }()), (0, _defineProperty2["default"])(_result, beforeName, before), (0, _defineProperty2["default"])(_result, afterName, after), _result);
      result = (0, _pcmliUmbrella.mergeExtraConfig)(result, extraProps, args);
      return result;
    },
    options: function options(ownProps) {
      var result = {
        refetchQueries: refetchQueries
      }; //if extraOptions is function then execute it passing the ownProps, else this is an object so just use it

      var xOption = _lodash["default"].isFunction(extraOptions) ? extraOptions({
        ownProps: ownProps
      }) : extraOptions;
      result = (0, _pcmliUmbrella.mergeExtraConfig)(result, xOption);
      return result;
    }
  };
  return (0, _hoc.graphql)(query, graphqlConfig);
};

exports.withMutation = withMutation;
//# sourceMappingURL=withMutation.js.map