"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useAppSetting = exports.useAppSettings = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _useList2 = require("../useList");

var _graphql = require("./graphql");

var _lodash = require("lodash");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

/***
 * Retrieve multiple settings
 * @param appSettingKeys
 * @param filter
 * @param tenantId
 * @param useList
 */
var useAppSettings = function useAppSettings(_ref) {
  var tenantId = _ref.tenantId,
      appSettingKeys = _ref.appSettingKeys,
      filter = _ref.filter,
      _ref$useList = _ref.useList,
      useList = _ref$useList === void 0 ? _useList2.useList : _ref$useList;

  //when the appSettingKeys is available, use it on filtering values
  if (!(0, _lodash.isEmpty)(appSettingKeys)) {
    filter = {
      key: {
        $in: appSettingKeys
      },
      'scope.key': tenantId
    };
  }

  var _useList = useList(_objectSpread(_objectSpread({}, _graphql.queries.singleConfig), {}, {
    limit: 0,
    filter: filter,
    options: {
      queryOptions: {
        skip: !tenantId && (0, _lodash.isEmpty)(appSettingKeys)
      }
    }
  })),
      settings = _useList.list,
      loading = _useList.loading,
      error = _useList.error;

  return {
    settings: settings,
    loading: loading,
    error: error
  };
};
/***
 * Retrieve and update single setting
 * @param tenantId
 * @param appSettingKey
 * @param filter
 * @param useList
 * @returns {undefined|{loading, error}}
 */


exports.useAppSettings = useAppSettings;

var useAppSetting = function useAppSetting(_ref2) {
  var tenantId = _ref2.tenantId,
      appSettingKey = _ref2.appSettingKey,
      filter = _ref2.filter,
      _ref2$useList = _ref2.useList,
      useList = _ref2$useList === void 0 ? _useList2.useList : _ref2$useList;

  var _useAppSettings = useAppSettings({
    tenantId: tenantId,
    appSettingKeys: appSettingKey ? [appSettingKey] : undefined,
    filter: filter,
    useList: useList
  }),
      settings = _useAppSettings.settings,
      loading = _useAppSettings.loading,
      error = _useAppSettings.error;

  return {
    setting: (0, _lodash.get)(settings, '[0]'),
    loading: loading,
    error: error
  };
};

exports.useAppSetting = useAppSetting;
//# sourceMappingURL=index.js.map