"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useFetch = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = _interopRequireDefault(require("react"));

var _pcmliUmbrella = require("pcmli.umbrella.uni-core");

var _lodash = require("lodash");

/**
 * A common hook for fetch URL, it wraps the Utils.fetch but included the loading and error states
 * @param url - the url that will be called
 * @param options -  fetch options that will be used
 * @param handleError -  a customer function to handle the error
 * @param handleResponse - a custom function on how to handle the response
 * @param raw - setting the to true, will the raw response from the fetch
 * @param fetch - the fetch implementation that will be used
 * @returns an array, with [function, {error, loading}] - include the states changes for the results
 */
var useFetch = function useFetch(url, options) {
  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      handleError = _ref.handleError,
      handleResponse = _ref.handleResponse,
      _ref$raw = _ref.raw,
      raw = _ref$raw === void 0 ? false : _ref$raw,
      fetch = _ref.fetch;

  var _React$useState = _react["default"].useState(null),
      _React$useState2 = (0, _slicedToArray2["default"])(_React$useState, 2),
      error = _React$useState2[0],
      setError = _React$useState2[1];

  var _React$useState3 = _react["default"].useState(false),
      _React$useState4 = (0, _slicedToArray2["default"])(_React$useState3, 2),
      loading = _React$useState4[0],
      setLoading = _React$useState4[1];

  var fetchNow = /*#__PURE__*/function () {
    var _ref2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
      var newOptions,
          finalOptions,
          _args = arguments;
      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              newOptions = _args.length > 0 && _args[0] !== undefined ? _args[0] : {};
              _context.prev = 1;
              setLoading(true); //merge the new options

              finalOptions = (0, _lodash.merge)({}, options, newOptions);
              _context.next = 6;
              return _pcmliUmbrella.Utils.fetch(url, finalOptions, {
                handleError: handleError,
                handleResponse: handleResponse,
                lean: raw,
                fetch: fetch
              });

            case 6:
              return _context.abrupt("return", _context.sent);

            case 9:
              _context.prev = 9;
              _context.t0 = _context["catch"](1);
              setError(_context.t0);

            case 12:
              _context.prev = 12;
              setLoading(false);
              return _context.finish(12);

            case 15:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[1, 9, 12, 15]]);
    }));

    return function fetchNow() {
      return _ref2.apply(this, arguments);
    };
  }();

  return [fetchNow, {
    error: error,
    loading: loading
  }];
};

exports.useFetch = useFetch;
//# sourceMappingURL=index.js.map