"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useList = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _client = require("@apollo/client");

var _pcmliUmbrella = require("pcmli.umbrella.uni-core");

var _buildFetchMoreQuery = require("../../common/buildFetchMoreQuery");

var _lodash = require("lodash");

var _useListAction = require("./useListAction");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

/**
 * Use this to query graphql using the umbrella defined query
 * @param query             - the graphql query (required)
 * @param entity            - the entity we are querying (required)
 * @param projection        - how we select data on the database for performance optimization
 * @param sort              - how we sort the list
 * @param limit             - if the limit null or zero, it mean it should get all records
 * @param skip              - the number of documents to be skip, if not defined/null will be set as zero
 * @param filter            - the filter condition on how we query the data
 * @param options           - should contain some options for apollo-related query
 * @param useQueryProxy
 * @param listActionOptions - pass the options if we are using the results from listActions
 */
var useList = function useList() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      query = _ref.query,
      entity = _ref.entity,
      projection = _ref.projection,
      sort = _ref.sort,
      limit = _ref.limit,
      skip = _ref.skip,
      filter = _ref.filter,
      _ref$options = _ref.options,
      options = _ref$options === void 0 ? {} : _ref$options,
      _ref$useQueryProxy = _ref.useQueryProxy,
      useQueryProxy = _ref$useQueryProxy === void 0 ? _client.useQuery : _ref$useQueryProxy,
      listActionOptions = _ref.listActionOptions;

  _pcmliUmbrella.Utils.requiredCheck(query, 'useList.query');

  _pcmliUmbrella.Utils.requiredCheck(entity, 'useList.entity');

  var _options$queryOptions = options.queryOptions,
      queryOptions = _options$queryOptions === void 0 ? {} : _options$queryOptions;
  var listActionResults = (0, _useListAction.useListAction)(listActionOptions); //the sorts and filters will come from the listActions if specified

  if (!(0, _lodash.isEmpty)(listActionOptions)) {
    sort = listActionResults === null || listActionResults === void 0 ? void 0 : listActionResults.sort;
    filter = listActionResults === null || listActionResults === void 0 ? void 0 : listActionResults.filter;
  } //because the filter and options are part of what apollo tracks, we need to have a consistent way of
  //stringify that, else it will query and do extra call, and this buildQueryVariables handles that


  var variables = (0, _pcmliUmbrella.buildQueryVariables)(skip, filter, {
    sort: sort,
    projection: projection
  }, limit); //use lodash merge so we can overwrite values to the variables

  queryOptions = (0, _lodash.merge)({
    variables: variables,
    notifyOnNetworkStatusChange: true,
    fetchPolicy: 'cache-and-network',
    nextFetchPolicy: 'cache-first'
  }, queryOptions); //execute the query here

  var queryResult = useQueryProxy(query, queryOptions);

  var _ref2 = queryResult || {},
      data = _ref2.data,
      fetchMore = _ref2.fetchMore,
      pageInfo = _ref2.pageInfo;

  var list = (0, _lodash.get)(data, entity, undefined); //get the pageInfo from server for backward compatibility, and if it's not available then use the new way

  pageInfo = pageInfo || (0, _lodash.get)(data, "".concat(entity, "Meta"));

  var result = _objectSpread(_objectSpread(_objectSpread({}, queryResult), {}, {
    pageInfo: pageInfo,
    list: list,
    dataArray: list,
    loadMoreEntries: function loadMoreEntries(skip, searchString, sortObject) {
      return (0, _buildFetchMoreQuery.buildFetchMoreQuery)(entity, fetchMore, skip, searchString, sortObject);
    }
  }, listActionResults), {}, {
    sort: sort,
    filter: filter
  });

  return result;
};

exports.useList = useList;
//# sourceMappingURL=index.js.map